from __future__ import annotations as _annotations

print("Examples:")
print("- examples/telegram_gateway.py")
print("- examples/pydantic_acp_agent.py")
print("- examples/langchain_acp_graph.py")
print("- examples/acprouter_with_acpkit_instance.py")
print("- examples/acpremote_bridge_server.py")
print("- examples/acprouter_remote_acp.py")
